<?php

namespace app\admin\service\order;

use app\model\GoodsRunning;
use app\model\GoodsSku;
use app\model\Member;
use app\model\Order;
use app\model\OrderSheet;
use app\model\PayDetail;
use app\model\SysDept;
use support\Db;
use support\exception\BusinessException;
use support\Log as SupportLog;
use support\Redis;
use Yansongda\Pay\Log;
use Yansongda\Pay\Pay;

class OrderService
{

    /**
     * @Desc 自动确认收货
     * @Author Gorden
     * @Date 2024/4/11 16:09
     *
     * @return void
     */
    public static function AutomaticReceipt()
    {
        try {
            Db::beginTransaction();
            $timeUnix = strtotime("-7 days");
            $orders = Order::where('order_status_system', 'SIGNED')
                ->where('order_addtimes', '<', $timeUnix)
                ->get();
            foreach ($orders as $order) {
                // 订单主表
                Order::where('order_id', $order->order_id)->update([
                    'order_is_complete' => 'Y',
                    'order_status_system' => 'CONFIRM',
                    'order_status_storage' => 'DONE'
                ]);
                // 订单详情表
                OrderSheet::where('join_sheet_order_id', $order->order_id)->update(['order_sheet_status' => 'DONE']);

                // 7天后自动完成 order_is_complete=Y
//                $redis = Redis::connection();
//                $key = Order::AUTOMATIC_COMPLETE_PREFIX . date('Ymd', strtotime("+7 days"));
//                $redis->sadd($key, $order->order_id);
            }
            Db::commit();
        } catch (\Exception $e) {
            Db::rollBack();
        }
    }

    /**
     * @Desc 自动完成订单
     * @Author Gorden
     * @Date 2024/7/16 9:37
     *
     * @return void
     */
    public static function AutomaticComplete()
    {
        Db::beginTransaction();
        try {

            $redis = Redis::connection();
            $key = Order::AUTOMATIC_COMPLETE_PREFIX . date('Ymd');
            $orderIds = $redis->smembers($key);

            foreach ($orderIds as $orderId) {
                $order = Order::where('order_id', $orderId)
                    ->select('order_is_complete', 'order_category', 'order_status_system')
                    ->first();
                if ($order && $order->order_is_complete != 'Y' && $order->order_category != 'RETURN' && in_array($order->order_status_system, ['RECVING', 'SIGNED', 'CONFIRM'])) {
                    // 更新主表
                    Order::where('order_id', $orderId)->update(['order_is_complete' => 'Y', 'order_status_system' => 'CONFIRM', 'order_status_storage' => 'DONE']);
                    // sheet表
                    OrderSheet::where('join_sheet_order_id', $orderId)->update(['order_sheet_status' => 'DONE']);
                }
            }

            $redis->del($key);

            Db::commit();
        } catch (\Exception $e) {
            dump($e->getMessage());
            Db::rollBack();
        }
    }

    public static function checkPayingOrder()
    {
        try {
            Db::beginTransaction();
            $timeUnix = strtotime("-30 minutes");
            $orders = Order::where('order_status_system', 'PAYING')
                ->where('order_category', '<>', 'DISHES')     // 点餐不自动取消
                // ->where('order_platform','<>','SYSTEM')     // 后台产生的订单不自动取消
                ->where(function ($query) {
                    $query->where('order_platform', '<>', 'SYSTEM')->orWhereNull('order_platform');
                })
                ->where('order_addtimes', '<', $timeUnix)
                ->get();
            foreach ($orders as $order) {
                // 订单主表
                Order::where('order_id', $order->order_id)->update([
                    'order_is_complete' => 'Y',
                    'order_status_system' => 'CANCEL',
                    'order_status_payment' => 'CANCEL'
                ]);
                $sheets = OrderSheet::where('join_sheet_order_id', $order->order_id)->get();
                foreach ($sheets as $sheet) {
                    // 还原库存
                    $goodsSku = GoodsSku::where('goods_sku_id', $sheet->join_sheet_goods_sku_id)->first();
                    if (!empty($goodsSku) && !empty($goodsSku->goods_sku_storage_json)) {
                        $skuStorageJson = json_decode($goodsSku->goods_sku_storage_json, true);
                        if (isset($skuStorageJson['storage']) && !empty($skuStorageJson['storage'])) {
                            $skuStorageJson['storage'] = $skuStorageJson['storage'] + $sheet->order_sheet_num;
                            $goodsSku->goods_sku_storage_json = json_encode($skuStorageJson);
                            $goodsSku->save();
                        }
                    }


                    $goodsRunning = GoodsRunning::where('join_running_goods_id', $sheet->join_sheet_goods_id)->first();
                    if (!empty($goodsRunning)) {
                        $goodsRunning->goods_running_storage = $goodsRunning->goods_running_storage + $sheet->order_sheet_num;
                        $goodsRunning->goods_running_sale = $goodsRunning->goods_running_sale - $sheet->order_sheet_num;
                        $goodsRunning->save();
                    }
                }

                // 订单详情表
                OrderSheet::where('join_sheet_order_id', $order->order_id)->update(['order_sheet_status' => 'CANCEL']);
                // 支付表
                $payDetail = PayDetail::where('join_pay_order_id', $order->order_groupby)->first();
                if (!empty($payDetail)) {
                    $payExtendJson = [];
                    if (!empty($payDetail->pay_extend_json)) {
                        $payExtendJson = json_decode($payDetail->pay_extend_json, true);
                    }
                    $payExtendJson['cancel_times'] = date('Y-m-d H:i:s');
                    PayDetail::where('join_pay_order_id', $order->order_groupby)->update([
                        'pay_status' => 'CANCEL',
                        'pay_extend_json' => json_encode($payExtendJson)
                    ]);
                }
            }

            Db::commit();
        } catch (\Exception $e) {
            Db::rollBack();
        }
    }

    public static function generateWriteOffData($params)
    {
        return [
            'charge' => [
                'charge_amount' => 1,
                'charge_content' => $params['order_remark'] ?? '',
                'charge_user_id' => $params['write_off_member_id'],
                'charge_premises' => $params['dept_premises_id'],
            ],
            'member_id' => $params['join_order_member_id']
        ];
    }

    public static function generateAppointmentApplyData($params)
    {
        $member = Member::with('cert', 'info')
            ->where('member_id', $params['join_order_member_id'])
            ->first();
        $name = '';
        if (!empty($member) && !empty($member->cert) && !empty($member->cert->member_cert_name)) {
            $name = $member->cert->member_cert_name;
        } else if (!empty($member) && !empty($member->info) && !empty($member->info->member_info_nickname)) {
            $name = $member->info->member_info_nickname;
        }

        return [
            'name' => $name,
            'times' => '',
            'mobile' => !empty($member) ? $member->member_mobile : '',
            'person' => $params['order_sheet_num'] ?? '',
            'premises' => $params['dept_premises_id'] ?? ''
        ];
    }

    /**
     * 微信支付宝扫码支付
     */
    public static function qrcodePay($params)
    {
        $log = SupportLog::channel('pay');
        $log->info("PAY_PARAMS", json_decode(json_encode($params), true));
        $params['order_amount_pay'] = floatval($params['order_amount_pay']);
        $qrcodeNbr = $params['qrcode_nbr'];
        $prefix = substr($qrcodeNbr, 0, 2);
        // 模拟数据
        // $result = [
        //     'return_code'=>'SUCCESS',
        //     'result_code' => 'SUCCESS'
        // ];
        // return $result;
        // 微信支付
        if (in_array($prefix, [10, 11, 12, 13, 14, 15])) {
            $payData = [
                'out_trade_no' => $params['orderGroupId'],
                'body' => '万悦康养订单',
                'total_fee' => $params['order_amount_pay'] * 100,
                'auth_code' => $params['qrcode_nbr'],
            ];
            try {
//                Pay::wechat(config('payment.wxpay'))->pos($payData);
                $result = self::findWxpay('OD202408161411135Z90KS', 0);
            } catch (\Exception $e) {
                $log->error("WXPAY", ['msg' => $e->getMessage()]);
                throw new BusinessException("支付失败");
            }

            $log->info("WXPAY_RETURN", json_decode(json_encode($result), true));

        } else if (in_array($prefix, [25, 26, 27, 28, 29, 30])) {
            $payData = [
                'out_trade_no' => $params['orderGroupId'],
                'total_amount' => $params['order_amount_pay'],
                'subject' => '万悦康养订单',
                'auth_code' => $params['qrcode_nbr'],
            ];
            try {
                Pay::alipay(config('payment.alipay'))->pos($payData);
                $result = self::findAlipay($params['orderGroupId'], 0);
            } catch (\Exception $e) {
                $log->error("ALIPAY", ['msg' => $e->getMessage()]);
                throw new BusinessException("支付失败");
            }

            $log->info("ALIPAY_RETURN", json_decode(json_encode($result), true));

        } else {
            throw new BusinessException("二维码无效");
        }

        return $result;
    }

    /**
     * @Desc 查询微信支付
     * @Author Gorden
     * @Date 2024/8/16 14:55
     *
     * @param $orderId
     * @param $nbr 循环次数
     * @return mixed|void
     * @throws BusinessException
     */
    public static function findWxpay($orderId, $nbr = 0)
    {
        if ($nbr >= 6) {
            throw new BusinessException("订单查询失败");
        }
        try {
            if($nbr > 2) {
                $result = Pay::wechat(config('payment.wxpay'))->find($orderId, 'pos');
                $result = json_decode(json_encode($result), true);
            }
        } catch (\Exception $e) {
            dump($e->getMessage());
            SupportLog::channel('pay')->error("WXPAY", ['msg' => $e->getMessage()]);
            throw new BusinessException("订单查询失败");
        }

        if (!empty($result['return_code']) && $result['return_code'] == 'SUCCESS' && !empty($result['result_code']) && $result['result_code'] == 'SUCCESS') {
            return $result;
        } else {
            dump("睡：".$nbr);
            sleep(5);
            self::findWxpay($orderId, $nbr + 1);
        }
    }

    /**
     * @Desc 查询支付宝订单
     * @Author Gorden
     * @Date 2024/8/16 15:16
     *
     * @param $orderId
     * @param $nbr 循环次数
     * @return mixed|void
     * @throws BusinessException
     */
    public static function findAlipay($orderId, $nbr = 0)
    {
        if ($nbr >= 6) {
            throw new BusinessException("订单查询失败");
        }
        try {
            if($nbr > 2){
                $result = Pay::alipay(config('payment.alipay'))->find($orderId);
                $result = json_decode(json_encode($result), true);
            }
        } catch (\Exception $e) {
            dump($e->getMessage());
            SupportLog::channel('pay')->error("ALIPAY", ['msg' => $e->getMessage()]);
            throw new BusinessException("订单查询失败");
        }

        if (!empty($result['code']) && $result['code'] == '10000' && !empty($result['trade_status']) && $result['trade_status'] == 'TRADE_SUCCESS') {
            return $result;
        } else {
            dump("睡：".$nbr);
            sleep(5);
            self::findAlipay($orderId, $nbr + 1);
        }
    }

    /**
     * 验证产品库存
     */
    public static function checkGoodsStorage($params)
    {
        foreach ($params['goodsContentList'] as $goods) {
            // 减库存,规格和总库存
            if (!isset($params['submit_goods_classify']) || $params['submit_goods_classify'] != 'MEALS') {
                $goodsSku = GoodsSku::where('goods_sku_id', $goods['sku_id'])->first();
                $skuStorageJson = json_decode($goodsSku->goods_sku_storage_json, true);
                if (isset($skuStorageJson['storage']) && !empty($skuStorageJson['storage'])) {
                    $skuStorageJson['storage'] = $skuStorageJson['storage'] - $goods['nbr'];
                }
                if (!isset($skuStorageJson['storage']) || (!empty($skuStorageJson['storage']) && $skuStorageJson['storage'] < 0)) {
                    throw new BusinessException('库存不足');
                }
            }
            $goodsRunning = GoodsRunning::where('join_running_goods_id', $goods['goods_id'])->first();
            $goodsRunning->goods_running_storage = $goodsRunning->goods_running_storage - $goods['nbr'];
            if ($goodsRunning->goods_running_storage < 0) {
                throw new BusinessException('库存不足');
            }
        }
    }

    /**
     * 下单时结算的组合支付payDetail
     */
    public static function createPayDetail($params)
    {
        $insertPayDetailData = [
            'join_pay_member_id' => $params['join_order_member_id'],
            'join_pay_order_id' => $params['orderGroupId'],
            'pay_status' => 'SUCCESS',
            'pay_category' => !empty($params['submit_goods_classify']) ? $params['submit_goods_classify'] : $params['goods_classify'],
            'pay_amount' => $params['order_amount_pay'],
            'pay_paytimes' => date('Y-m-d H:i:s'),
            'pay_prepayid' => $params['pay_category'],
            'pay_json_request' => json_encode($params),
            'pay_json_response' => $params['pay_json_response'] ?? '[]',
            'pay_addtimes' => time()
        ];

        PayDetail::insert($insertPayDetailData);
    }

    /**
     * 组合支付，PayDetail
     */
    public static function createPayConstituteDetail($params, $payDetail)
    {
        $qrcodePrepayId = '';
        if (in_array($params['pay_category'], ['WXPAY', 'ALIPAY'])) {
            $qrcodePrepayId = $params['join_order_member_id'] . '-QRCODE';
        }
        if (!$payDetail || ($payDetail->pay_prepayid != $params['pay_category'] && $payDetail->pay_prepayid != $qrcodePrepayId)) {
            $payDetail = new PayDetail();
            $payDetail->join_pay_member_id = $params['join_order_member_id'];
            $payDetail->join_pay_order_id = $params['orderGroupId'];
            $payDetail->pay_category = $params['goods_classify'] ?? '';
            $payDetail->pay_prepayid = $params['pay_category'];
            $payDetail->pay_json_request = json_encode($params);
            $payDetail->pay_json_response = $params['pay_json_response'] ?? json_encode([
                'pay-result' => '支付成功', 'result-datetime' => date('Y-m-d H:i:s')
            ]);
            $payDetail->pay_addtimes = time();
        }
        if ($payDetail->pay_prepayid == $qrcodePrepayId) {
            $payDetail->pay_prepayid = $params['pay_category'];
        }
        $payDetail->pay_json_response = $params['pay_json_response'] ?? json_encode([
            'pay-result' => '支付成功', 'result-datetime' => date('Y-m-d H:i:s')
        ]);
        $payDetail->pay_amount = $params['order_amount_pay'];
        $payDetail->pay_paytimes = date('Y-m-d H:i:s');
        $payDetail->pay_status = 'SUCCESS';
        $payDetail->save();
    }
}